#include "kosSyst.h"
#include "KosFile.h"
#include "gfxdef.h"
#include "crc32.h"

extern "C" void __stdcall lzma_decompress(
	 const void* source,
	 void* destination,
	 unsigned dest_length);

void jpeg_decompress(
	 const void* source,
	 unsigned source_length,
	 void* destination,
	 unsigned dest_length)
{
	Byte* ipcmem = new Byte[16+8+dest_length];
	((Dword*)ipcmem)[0] = 0;
	((Dword*)ipcmem)[1] = 8;
	__asm
	{
		mov	eax,40
		mov ebx,EM_IPC
		int 40h
		mov eax,60
		mov ebx,1
		mov ecx,ipcmem
		mov edx,dest_length
		add edx,16+8
		int 40h
	}
	char param[10];
	sprintf(param,"\x01%U",source_length);
	//kosFileInfo fi = {7,0,(Dword)param,0,0,"/rd/1/jpegview"};
	kosFileInfo fi;
	fi.rwMode = 7;
	fi.OffsetLow = 0;
	fi.OffsetHigh = (Dword)param;
	fi.dataCount = 0;
	fi.bufferPtr = 0;
	strcpy(fi.fileURL,"/sys/jpegview");
	int pid = kos_FileSystemAccess(&fi);
	if (pid <= 0)
	{
l3:
		rtlDebugOutString("Cannot launch jpegview or jpegview error");
		kos_ExitApp();
	}
	__asm
	{
		mov edi,10
l1:
		mov eax,60
		mov ebx,2
		mov ecx,pid
		mov edx,source
		mov esi,source_length
		int 40h
		test eax,eax
		jz l2
		dec edi
		jz l3
		mov eax,5
		mov ebx,eax
		int 40h
		jmp l1
l2:
		mov eax,23
		mov ebx,500
		int 40h
		test eax,eax
		jz l3
		mov eax,60
		mov ebx,1
		xor ecx,ecx
		xor edx,edx
		int 40h
	}
	memcpy(destination,(char*)ipcmem+16+8,dest_length);
	delete [] ipcmem;
}

//////  CKosBitmap

CKosBitmap::CKosBitmap()
{
	this->bmpID = -1;
	this->buffer = NULL;
	this->sizeX = 0;
	this->sizeY = 0;
}


CKosBitmap::~CKosBitmap()
{
	if ( this->buffer != NULL ) delete this->buffer;
}


//    
bool CKosBitmap::LoadFromArch( SCompBmpHeader *bmpArchDesc, CKosFile *fromFile, int ID )
{
	Byte *tmpBuff;
	
	//
	if ( this->buffer != NULL )
	{
		delete this->buffer;
		this->buffer = NULL;
	}
	//
	this->buffer = new RGB[bmpArchDesc->sizeX * bmpArchDesc->sizeY];
	//
	tmpBuff = new Byte[bmpArchDesc->compressedSize];
	//
	fromFile->Seek( SEEK_SET, bmpArchDesc->physicalOffset );
	if ( fromFile->Read( tmpBuff, bmpArchDesc->compressedSize ) == bmpArchDesc->compressedSize )
	{
		//
		if ( bmpArchDesc->compressedSize == bmpArchDesc->uncompressedSize+1)
		{
			// JPEG image
			jpeg_decompress( tmpBuff, bmpArchDesc->compressedSize,
				this->buffer, bmpArchDesc->sizeX * bmpArchDesc->sizeY * 3);
		}
		else if ( bmpArchDesc->compressedSize != bmpArchDesc->uncompressedSize )
		{
			// LZMA-packed BMP
			lzma_decompress( tmpBuff, this->buffer, bmpArchDesc->uncompressedSize);
		}
		else
		{
			//
			memcpy( (Byte *)(this->buffer), tmpBuff, bmpArchDesc->compressedSize );
		}
		//
		this->sizeX = bmpArchDesc->sizeX;
		this->sizeY = bmpArchDesc->sizeY;
		this->bmpID = ID;
	}
	//
	delete tmpBuff;
	//
	return true;
}


//    
void CKosBitmap::Draw( Word x, Word y )
{
	//
	if ( this->buffer != NULL )
		//
		kos_PutImage( this->buffer, this->sizeX, this->sizeY, x, y );
}


//     
RGB * CKosBitmap::GetBits()
{
	return this->buffer;
}


//   
void CKosBitmap::GetSize( Word &cx, Word &cy )
{
	cx = this->sizeX;
	cy = this->sizeY;
}

//    
void CKosBitmap::Scale(Word size, RGB *mainBits)
{
	buffer = new RGB[(sizeX=blockSize)*(sizeY=blockSize*11)];
	memset((Byte*)buffer,0,3*blockSize*blockSize*11);
	RGB* tmpBuf = new RGB[blockSize*size];
	for (int k=0;k<11;k++)
	{
		int delta = (blockSize - size)/2;
		int i,j;
		int a;
		int d1 = blockSize/size;
		int d2 = (blockSize-d1*(size))*256/size;
		//   
		RGB* ptrBuf = tmpBuf;
		for (j=0;j<blockSize;j++)
		{
			RGB* srcBits = mainBits + blockSize*blockSize*(k+1) + blockSize*j;
			a = 0;
			for (i=0;i<size;i++)
			{
				ptrBuf->b = srcBits->b + (srcBits[1].b-srcBits[0].b)*a/256;
				ptrBuf->g = srcBits->g + (srcBits[1].g-srcBits[0].g)*a/256;
				ptrBuf->r = srcBits->r + (srcBits[1].r-srcBits[0].r)*a/256;
				ptrBuf++;
				srcBits += d1;
				a += d2;
				if (a >= 256)
				{
					a -= 256;
					srcBits++;
				}
			}
		}
		//   
		for (j=0;j<size;j++)
		{
			ptrBuf = buffer + blockSize*blockSize*k + blockSize*delta + delta+j;
			RGB* srcBits = tmpBuf + j;
			a = 0;
			for (i=0;i<size;i++)
			{
				ptrBuf->b = srcBits->b + (srcBits[size].b-srcBits[0].b)*a/256;
				ptrBuf->g = srcBits->g + (srcBits[size].g-srcBits[0].g)*a/256;
				ptrBuf->r = srcBits->r + (srcBits[size].r-srcBits[0].r)*a/256;
				ptrBuf += blockSize;
				srcBits += d1*size;
				a += d2;
				if (a >= 256)
				{
					a -= 256;
					srcBits += size;
				}
			}
		}
	}
	delete tmpBuf;
}


////////////////////// CFishka ///////////////////////

CFishka::CFishka( CKosBitmap *fromBmp, int yOffset, RGB insColour )
{
	int i, c;

	//
	this->bits = fromBmp->GetBits() + (yOffset * blockSize);
	this->transColour = insColour;
	//
	this->highLighted = new RGB[blockSize * blockSize];
	//
	for ( i = 0; i < (blockSize * blockSize); i++ )
	{
		//
		this->highLighted[i] = this->bits[i];
		//
		if ( this->highLighted[i] != this->transColour )
		{
			c = ( this->highLighted[i].b * 185 ) / 100;
			this->highLighted[i].b = (c > 255) ? 255 : c;
			c = ( this->highLighted[i].g * 185 ) / 100;
			this->highLighted[i].g = (c > 255) ? 255 : c;
			c = ( this->highLighted[i].r * 185 ) / 100;
			this->highLighted[i].r = (c > 255) ? 255 : c;
		}
	}
}

//
CFishka::~CFishka()
{
	//
	delete this->highLighted;
}


//
RGB * CFishka::GetBits()
{
	return this->bits;
}

//
RGB * CFishka::GetHighlightedBits()
{
	return this->highLighted;
}



